/***************************************************************************************
 * @file     board.h
 * @version  V1.0
 * @date     18 January 2018
 *
 * @note
 * VORAGO Technologies
 *
 * @note
 * Copyright (c) 2013-2018 VORAGO Technologies.
 *
 * @par
 * BY DOWNLOADING, INSTALLING OR USING THIS SOFTWARE, YOU AGREE TO BE BOUND BY
 * ALL THE TERMS AND CONDITIONS OF THE VORAGO TECHNOLOGIES END USER LICENSE AGREEMENT.
 * THIS SOFTWARE IS PROVIDED "AS IS". NO WARRANTIES, WHETHER EXPRESS, IMPLIED
 * OR STATUTORY, INCLUDING, BUT NOT LIMITED TO, IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. VORAGO TECHNOLOGIES
 * SHALL NOT, IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL, OR CONSEQUENTIAL
 * DAMAGES, FOR ANY REASON WHATSOEVER.
 *
 ****************************************************************************************/


#ifndef __BOARD_H
#define __BOARD_H

#include "va108xx.h"

/* RH_OBC1_PINS */
/* When defined, sets the UART0 pins to the pins connecting to the FTDI chip
   on the RH-OBC1 board. OBC1 uses F-RAM also. Undefine for REB1 */
//#define RH_OBC1_PINS

#define SOFTWARE_VERSION  "2018_11_14_v1"

//missing from our design environment
#define STAT_SEL_PWMA      (0x3)
#define STAT_SEL_PWMB      (0x4)
#define FUNSEL0 0
#define FUNSEL1 1
#define FUNSEL2 2
#define FUNSEL3 3

#undef VOR_PORTA
#undef VOR_PORTB
#define VOR_PORTA           (VOR_GPIO->BANK[0])
#define VOR_PORTB           (VOR_GPIO->BANK[1])

#define VOR_UART0           (VOR_UART->BANK[0])
#define VOR_UART1           (VOR_UART->BANK[1])

#define VOR_SPI0            (VOR_SPI->BANK[0])
#define VOR_SPI1            (VOR_SPI->BANK[1])

/* System Clock in Hz. */
#define SYSCLK	(50000000)
#define configUART_BAUDRATE (115200)

/* Timer that generates second tick. */
#define VOR_TIM_SEC_INDEX   (19)
#define VOR_TIM_SEC         (VOR_TIM->BANK[VOR_TIM_SEC_INDEX])
#define VOR_IRQ_SEC_INDEX 		((IRQn_Type)27)

///debug dimer
#define VOR_TIM_DBG_INDEX   (20)
#define VOR_TIM_DBG         (VOR_TIM->BANK[VOR_TIM_DBG_INDEX])
#define VOR_IRQ_TIM_DBG_INDEX 		((IRQn_Type)28)

#define VOR_SPIB_NUM         (1)

#define VOR_TIM_SEQ        (VOR_TIM1)
#define VOR_TIM_SEQ_NUM    (1)
#define VOR_IRQ_TIM_SEQ_NUM    (27)

#ifdef RH_OBC1_PINS
#define WD_PORT   PORTA
#define WD_BANK   VOR_GPIO->BANK[0]
#define WD_PIN    (24)

#define UARTA_TX_PORT        PORTA
#define UARTA_TX_BANK        VOR_GPIO->BANK[0]
#define UARTA_TX_PIN         (17)
#define UARTA_TX_FUNSEL      (FUNSEL3)

#define UARTA_RX_PORT        PORTA
#define UARTA_RX_BANK        VOR_GPIO->BANK[0]
#define UARTA_RX_PIN         (16)
#define UARTA_RX_FUNSEL      (FUNSEL3)
#else
#define UARTA_TX_PORT        PORTA
#define UARTA_TX_BANK        VOR_GPIO->BANK[0]
#define UARTA_TX_PIN         (31)
#define UARTA_TX_FUNSEL      (FUNSEL3)
#define UARTA_BAUDRATE       (configUART_BAUDRATE)

#define UARTA_RX_PORT        PORTA
#define UARTA_RX_BANK        VOR_GPIO->BANK[0]
#define UARTA_RX_PIN         (30)
#define UARTA_RX_FUNSEL      (FUNSEL3)

#define MAX11619_CS_N_PORT   PORTA
#define MAX11619_CS_N_BANK   VOR_GPIO->BANK[0]
#define MAX11619_CS_N_PIN    (17)

#define LED_D2_PORT          PORTA
#define LED_D2_BANK          VOR_GPIO->BANK[0]
#define LED_D2_PIN           (10)

#define LED_D3_PORT          PORTA
#define LED_D3_BANK          VOR_GPIO->BANK[0]
#define LED_D3_PIN           (7)

#define LED_D4_PORT          PORTA
#define LED_D4_BANK          VOR_GPIO->BANK[0]
#define LED_D4_PIN           (6)
#endif

#define VOR_UART0_IRQ_INDEX		((IRQn_Type)3)
#define VOR_UART1_IRQ_INDEX		((IRQn_Type)4)

#ifdef RH_OBC1_PINS
//Boot Rom CYP15B102
//Dedicated ROM port, no FUNSEL/DIR possible
//CS_N can be redirected, =0 is dedicated pin
#define CYP15B102_SPI_NUM   (2)  //ROM port
#define CYP15B102_CS_N_NUM  (0)
#endif

/*****************************************************************************/
/* Variable declarations                                                     */
/*****************************************************************************/

/* Alias for GPIO Ports. */
static VOR_GPIO_PERIPHERAL_Type *PORTA = &(VOR_GPIO->BANK[0]);
static VOR_GPIO_PERIPHERAL_Type *PORTB = &(VOR_GPIO->BANK[1]);

/*****************************************************************************/ 
/* Global function prototypes ('extern', definition in C source)             */ 
/*****************************************************************************/

#if defined ( __CC_ARM )
/* watchdog feed */
__INLINE void WDFeed(){
#ifdef RH_OBC1_PINS
  WD_BANK.TOGOUT=1UL<<WD_PIN;
#endif
}
#elif defined ( __ICCARM__ )
extern __INLINE void VOR_GPIO_PinMux(VOR_GPIO_PERIPHERAL_Type *port, uint32_t pin, uint32_t funsel);
extern __INLINE void WDFeed(void);
#endif

/*****************************************************************************/ 
/* End of file                                                               */ 
/*****************************************************************************/
#endif // __BOARD_H

