/***************************************************************************************
 * @file     main.c
 * @version  V1.0
 * @date     31 March 2018
 *
 * @note
 * VORAGO Technologies
 *
 * @note
 * Copyright (c) 2013-2016 VORAGO Technologies.
 *
 * @par
 * BY DOWNLOADING, INSTALLING OR USING THIS SOFTWARE, YOU AGREE TO BE BOUND BY
 * ALL THE TERMS AND CONDITIONS OF THE VORAGO TECHNOLOGIES END USER LICENSE AGREEMENT.
 * THIS SOFTWARE IS PROVIDED "AS IS". NO WARRANTIES, WHETHER EXPRESS, IMPLIED
 * OR STATUTORY, INCLUDING, BUT NOT LIMITED TO, IMPLIED WARRANTIES OF MERCHANTABILITY
 * AND FITNESS FOR A PARTICULAR PURPOSE APPLY TO THIS SOFTWARE. VORAGO TECHNOLOGIES
 * SHALL NOT, IN ANY CIRCUMSTANCES, BE LIABLE FOR SPECIAL, INCIDENTAL, OR CONSEQUENTIAL
 * DAMAGES, FOR ANY REASON WHATSOEVER.
 *
 ****************************************************************************************/
 
/*****************************************************************************/ 
/* Include files                                                             */ 
/*****************************************************************************/

#include <stdint.h>
#include <stdbool.h>
#include <stdio.h>
#include "va108xx.h"
#include "board.h"
#include "uart.h"
#include "bootloader.h"

/*****************************************************************************/ 
/* Global variable definitions (declared in header file with 'extern')       */ 
/*****************************************************************************/

/* Alias for GPIO Ports. */
extern VOR_GPIO_PERIPHERAL_Type *PORTA;
extern VOR_GPIO_PERIPHERAL_Type *PORTB;

volatile uint32_t u32SecondsCounter=0;
volatile uint32_t u32SecToggle=0;
volatile uint32_t u32MsCounter=0;
volatile uint32_t u32MsToggle=0;

/*****************************************************************************/ 
/* Function implementation - global ('extern') and local ('static')          */ 
/*****************************************************************************/

__INLINE uint32_t NVIC_checkIRQ(uint32_t IRQn)
{
  return (NVIC->ISPR[0U] & (1UL << IRQn));
}

__INLINE void NVIC_clrPendingIRQ(uint32_t IRQn)
{
  NVIC->ICPR[0U] = (1UL << IRQn);
}

static void VOR_GPIOA_PinMux(uint32_t pin, uint32_t funsel) {
		VOR_IOCONFIG->PORTA[pin] = (VOR_IOCONFIG->PORTA[pin] & ~((0x3)<<IOCONFIG_PORTA_FUNSEL_Pos))	| ((funsel)<<IOCONFIG_PORTA_FUNSEL_Pos);
}
static void VOR_GPIOB_PinMux(uint32_t pin, uint32_t funsel) {
		VOR_IOCONFIG->PORTB[pin] = (VOR_IOCONFIG->PORTB[pin] & ~((0x3)<<IOCONFIG_PORTB_FUNSEL_Pos))	| ((funsel)<<IOCONFIG_PORTB_FUNSEL_Pos);
} 

__INLINE void VOR_GPIO_PinMux(VOR_GPIO_PERIPHERAL_Type *port, uint32_t pin, uint32_t funsel) {
  if(port==PORTA)
	  VOR_GPIOA_PinMux(pin,funsel);
	else
		VOR_GPIOB_PinMux(pin,funsel);
}

/* watchdog feed */
#if defined ( __ICCARM__ )
__INLINE void WDFeed(){
#ifdef RH_OBC1_PINS
  WD_BANK.TOGOUT=1UL<<WD_PIN;
#endif
}
#endif

void SecTimerInit(void)
{ 
    /* Configure Timer to generate interupt every second, for internal
       timekeeping. */
	  VOR_SYSCONFIG->TIM_CLK_ENABLE |= (1UL << VOR_TIM_SEC_INDEX);
    VOR_TIM_SEC.CTRL = (TIM_PERIPHERAL_CTRL_IRQ_ENB_Msk |
                        (STAT_SEL_PWMA << TIM_PERIPHERAL_CTRL_STATUS_SEL_Pos));
    VOR_TIM_SEC.RST_VALUE = SYSCLK;  
    VOR_TIM_SEC.CNT_VALUE = 0;
    
    VOR_IRQSEL->TIM[VOR_TIM_SEC_INDEX] = VOR_IRQ_SEC_INDEX;
    NVIC_SetPriority(VOR_IRQ_SEC_INDEX, 1);
    NVIC_EnableIRQ(VOR_IRQ_SEC_INDEX);
    
    VOR_TIM_SEC.ENABLE = 1;
}

void MsTimerInit(void)
{ 	
		/* Configure Timer to generate interupt every millisecond, for internal
       timekeeping. */
	  VOR_SYSCONFIG->TIM_CLK_ENABLE |= (1UL << VOR_TIM_DBG_INDEX);
    VOR_TIM_DBG.CTRL = (TIM_PERIPHERAL_CTRL_IRQ_ENB_Msk |
                        (STAT_SEL_PWMA << TIM_PERIPHERAL_CTRL_STATUS_SEL_Pos));
    VOR_TIM_DBG.RST_VALUE = SYSCLK/1000;//1ms
    VOR_TIM_DBG.CNT_VALUE = 0;
    
    VOR_IRQSEL->TIM[VOR_TIM_DBG_INDEX] = VOR_IRQ_TIM_DBG_INDEX;
    NVIC_SetPriority(VOR_IRQ_TIM_DBG_INDEX, 1);
    NVIC_EnableIRQ(VOR_IRQ_TIM_DBG_INDEX);
    
    VOR_TIM_DBG.ENABLE = 1;
}


/*******************************************************************************
 **
 ** @brief  Enables weak pulldowns on all pins.
 ** Sets WDT direction
 **
 ******************************************************************************/
void ConfigureIO(void)
{
	  uint32_t pin;
	
#ifdef RH_OBC1_PINS
    //Watchdog
    WD_BANK.DIR   |=1UL<<WD_PIN;
	  WD_BANK.TOGOUT =1UL<<WD_PIN;
#endif
	 
	  //Create default bus condition if FUNSEL moves
	  UARTA_TX_BANK.DIR   |=1UL<<UARTA_TX_PIN;
	  UARTA_TX_BANK.SETOUT =1UL<<UARTA_TX_PIN;

	  VOR_GPIO_PinMux(UARTA_RX_PORT, UARTA_RX_PIN, UARTA_RX_FUNSEL);  	
	  VOR_GPIO_PinMux(UARTA_TX_PORT, UARTA_TX_PIN, UARTA_TX_FUNSEL);
		
		//default to high weak pull on uartrx pin
		//had to hardcode PORTA here
		VOR_IOCONFIG->PORTA[UARTA_RX_PIN] |= IOCONFIG_PORTA_PLEVEL_Msk;
	 
	  // Enable pulldowns on all pins
    for(pin = 0; pin < 32; pin++)
    {
        VOR_IOCONFIG->PORTA[pin] |= IOCONFIG_PORTA_PEN_Msk;
        VOR_IOCONFIG->PORTB[pin] |= IOCONFIG_PORTB_PEN_Msk;
    }
		
#ifdef RH_OBC1_PINS
    // for OBC1 board
		VOR_PORTA.DIR |= 1UL<<22;
		VOR_PORTA.DIR |= 1UL<<23;
		VOR_PORTA.SETOUT=1UL<<22; // CSN high
		VOR_PORTA.SETOUT=1UL<<23; // CSN high
#else
    // REB1
    // set ADC CS_N high
    MAX11619_CS_N_BANK.DIR   |= 1UL<<MAX11619_CS_N_PIN;
    MAX11619_CS_N_BANK.SETOUT = 1UL<<MAX11619_CS_N_PIN;
    
    // LED pins outputs
    LED_D2_BANK.DIR |= 1UL<<LED_D2_PIN;
    LED_D3_BANK.DIR |= 1UL<<LED_D3_PIN;
    LED_D4_BANK.DIR |= 1UL<<LED_D4_PIN;
#endif
}
 
 /*******************************************************************************
 **
 ** @brief  Cause a wait for a specified number of milliseconds
 **
 ******************************************************************************/
void waitMs(uint32_t ms)
{
  uint32_t timeout = u32MsCounter + ms;
	while(u32MsCounter != timeout)
  {
	  /* Feed watchdog. */
		WDFeed();
  }
}
 
/*******************************************************************************
 **
 ** @brief  Application entry point.
 **
 ******************************************************************************/
int main(void)
{
	bool enterBootloader = false;

#ifdef PC_MODE
#define UART_BUF_SZ 64
	char strBufTx[UART_BUF_SZ];
#endif
	
  /* Enable clocks. */
 	VOR_SYSCONFIG->PERIPHERAL_CLK_ENABLE =
        (CLK_ENABLE_PORTA | CLK_ENABLE_PORTB | CLK_ENABLE_GPIO |
         CLK_ENABLE_IOCONFIG | CLK_ENABLE_PORTIO | CLK_ENABLE_SYSTEM | 
	       CLK_ENABLE_IRQSEL );


  /* Configure IO. */
  ConfigureIO();

	UART0Init();
	SecTimerInit();
	MsTimerInit();
	
#ifdef PC_MODE
	sprintf(strBufTx,"\r\nVA108xx Bootloader Version %s\r\n", SOFTWARE_VERSION);
	UART0TxStr((char *)strBufTx);
	sprintf(strBufTx,"CPUID = 0x%08x\r\n" , VOR_SYSCONFIG->EF_ID);
	UART0TxStr((char *)strBufTx);
  sprintf(strBufTx,"RST_STAT = 0x%08x\r\n" , VOR_SYSCONFIG->RST_STAT);
	UART0TxStr((char *)strBufTx);
	UART0TxStr("Press spacebar to enter bootloader command");
#endif

  /* for timeout period, looks for heralding character */
	u32MsCounter = 0;
	while(u32MsCounter < BOOTLOADER_MSEC_TIMEOUT)
	{
	  if (UART0IsRxByteAvailable())
	  {
			if(UART0RxByte() == BOOTLOADER_HERALDING_CHAR)
			{
				/* caught enter bootloader cmd */
				enterBootloader = true;
#ifndef PC_MODE
			  /* send response to heralding char */
				UART0TxByte(BOOTLOADER_HERALDING_RESP_CHAR);
#endif
				break;
			}
	  }
		waitMs(10);
		
#ifdef PC_MODE		
		if ((u32MsCounter % 1000) == 0)
		{
			UART0TxStr(".");
		}
#endif
		
		/* Feed watchdog. */ 
		WDFeed();
  }
	
	if(false == enterBootloader)
	{
		/* no hearalding char heard - attempt to run application */
		if(Bootloader_CheckAppIsValid())
		{
#ifdef PC_MODE
			UART0TxStr("\r\nRunning Application\r\n");
			waitMs(10);
#endif
			Bootloader_RunApp();
		}
		else
		{
#ifdef PC_MODE
			UART0TxStr("\r\nApplication FW image invalid\r\n");
#endif
		}
	}
	
	// bootloader loop - wait for code upload
  while(1)
  {		
		if(u32SecToggle)
		{
			u32SecToggle = 0;  
#ifndef RH_OBC1_PINS
      // reb1 toggle LED
			LED_D2_BANK.TOGOUT=1UL<<LED_D2_PIN;
#endif
		}
		
		Bootloader_Task();
		
		/* Feed watchdog. */
		WDFeed();
  }
}   // main

/*******************************************************************************
 **
 ** @brief  The ISR for tracking time.
 **
 ** Counts time in number of seconds.
 **
 ******************************************************************************/
void
OC27_IRQHandler(void)
{
  ++u32SecondsCounter;
	u32SecToggle=1;
}

void
OC28_IRQHandler(void)
{
  ++u32MsCounter;
	u32MsToggle=1;
}

/*****************************************************************************/ 
/* End of file                                                               */ 
/*****************************************************************************/
